@@ -22,13 +22,15 @@
 nft = function(## data
                x.train, times, delta=NULL, 
                x.test=matrix(nrow=0, ncol=0),
+               rm.const=TRUE, rm.dupe=TRUE,
                impute.bin=NULL, impute.prob=NULL,
                ## multi-threading
-               tc=1, ##OpenMP thread count
+               tc=getOption("mc.cores", 1), ##OpenMP thread count
                ##MCMC
                nskip=1000, ndpost=2000, 
-               nadapt=1000, adaptevery=100, 
-               chv = cor(x.train, method="spearman"),
+               nadapt=1000, adaptevery=100,
+               chv = NULL,
+               method="spearman", use="pairwise.complete.obs",
                pbd=c(0.7, 0.7), pb=c(0.5, 0.5),
                stepwpert=c(0.1, 0.1), probchv=c(0.1, 0.1),
                minnumbot=c(5, 5),
@@ -39,33 +41,55 @@ nft = function(## data
                k=5, sigmaf=NA,
                dist='weibull', 
                ## s function
-               sigmav=NULL, total.lambda=NA, total.nu=10, ##total.q=0.75,
+               sigmav=NULL, total.lambda=NA, total.nu=10, mask=NULL,
                ## survival analysis 
-               K=100, events=NULL, 
+               K=100, events=NULL, TSVS=FALSE,
                ##impute.mult=NULL, impute.prob=NULL, impute.miss=NULL,
                ## DPM LIO
                drawDPM=1L, 
                alpha=1, alpha.a=1, alpha.b=0.1, alpha.draw=1,
                neal.m=2, constrain=1, 
                m0=0, k0.a=1.5, k0.b=7.5, k0=1, k0.draw=1,
-               a0=1.5, b0.a=0.5, b0.b=1, b0=1, b0.draw=1,
+               ##a0=1.5, b0.a=0.5,
+               a0=3, b0.a=2, b0.b=1, b0=1, b0.draw=1,
                ## misc
-               printevery=100
+               na.rm=FALSE, probs=c(0.025, 0.975), printevery=100,
+               transposed=FALSE
                )
 {
     n=length(times)
     if(length(delta)==0) delta=rep(1, n)
     if(length(sigmav)==0) sigmav=rep(1, n)
-    x = x.train
-    xp = x.test
-    ##if(!transposed) {
+
+    if(!transposed) {
+        np=nrow(x.test)
+        if(np>0) {
+            x.=bMM(rbind(x.train, x.test), numcut=numcut, rm.const=rm.const, rm.dupe=rm.dupe,
+                   method=method, use=use)
+            x =cbind(x.$X[1:n, ])
+            xp=cbind(x.$X[n+(1:np), ])
+        } else {
+            x.=bMM(x.train, numcut=numcut, rm.const=rm.const, rm.dupe=rm.dupe,
+                   method=method, use=use)
+            x=x.$X
+            xp=x.test
+        }
+        xicuts=x.$xicuts
+        chv = x.$chv
         impute=CDimpute(x.train=cbind(x),
                         x.test=cbind(xp),
                         impute.bin=impute.bin)
         x=t(impute$x.train)
         xp=t(impute$x.test)
         transposed=TRUE
-    ##}
+    } else {
+        x =x.train
+        xp=x.test
+        np=ncol(xp)
+        if(length(chv)==0) chv=cor(t(x), use=use, method=method)
+        if(length(xicuts)==0) 
+            xicuts=xicuts(x, transposed=transposed, numcut=numcut)
+    }
 
     take.logs = (dist!='extreme')
     if(length(events)==0 && K>0) {
@@ -79,12 +103,7 @@ nft = function(## data
     nc=numcut
     
     p=nrow(x)
-    np=ncol(xp)
-
-    if(length(xicuts)==0) {
-        xicuts=xicuts(x, transposed=transposed, numcut=numcut)
-        ##return(xicuts)
-    }
+    ##np=ncol(xp)
 
     df = data.frame(times, delta, t(x))
     ##type='interval' not supported by survreg
@@ -225,7 +244,45 @@ if(K>0) {
     res$take.logs=take.logs
 }
     
+    s.train.max.=NULL
+    if(length(mask)==1 && 0<mask && mask<1) {
+        res$s.train[is.na(res$s.train)] = 0
+        s.train.min =apply(res$s.train, 1, quantile, probs=0)
+        s.train.max =apply(res$s.train, 1, quantile, probs=1)
+        s.train.max.=quantile(s.train.max, probs=mask)
+        s.train.mask=(0<s.train.min & s.train.max<=s.train.max.)
+        nd=sum(s.train.mask)
+        s.train.mask=which(s.train.mask)
+        res$f.train=res$f.train[s.train.mask, ]
+        res$s.train=res$s.train[s.train.mask, ]
+        res$z.train=res$z.train[s.train.mask, ]
+    } else s.train.mask=1:nd 
+    s.train.burn=c(1:burn, s.train.mask)
+
+    summarystats=(ndpost>=2)
+    if(summarystats) {
+        res$f.varcount[2:ndpost, ]=cbind(res$f.varcount[2:ndpost, ]-res$f.varcount[1:(ndpost-1), ])
+        res$f.varcount=cbind(res$f.varcount[s.train.mask, ])
+        dimnames(res$f.varcount)[[2]]=names(xicuts)
+        res$f.varcount.mean=apply(res$f.varcount, 2, mean)
+        res$f.varprob=res$f.varcount.mean/sum(res$f.varcount.mean)
+        res$s.varcount[2:ndpost, ]=cbind(res$s.varcount[2:ndpost, ]-res$s.varcount[1:(ndpost-1), ])
+        res$s.varcount=cbind(res$s.varcount[s.train.mask, ])
+        dimnames(res$s.varcount)[[2]]=names(xicuts)
+        res$s.varcount.mean=apply(res$s.varcount, 2, mean)
+        res$s.varprob=res$s.varcount.mean/sum(res$s.varcount.mean)
+    }
+
+    if(TSVS) return(res)
+    
     if(drawDPM>0) {
+        res$dpalpha=res$dpalpha[s.train.mask]
+        res$dpn=res$dpn[s.train.burn]
+        res$dpmu=res$dpmu[s.train.mask, ]
+        res$dpsd=res$dpsd[s.train.mask, ]
+        res$dpmu.=res$dpmu.[s.train.mask, ]
+        res$dpsd.=res$dpsd.[s.train.mask, ]
+        res$dpC=res$dpC[s.train.mask, ]
         if(burn>0) res$dpn.=res$dpn[-(1:burn)]
         else res$dpn.=res$dpn
     }
@@ -239,13 +296,15 @@ if(K>0) {
     res$MSE=mean(res$e.train.mean^2)
         res$s.train.mean=apply(res$s.train, 2, mean)
         res$ssd=sqrt(mean(res$s.train.mean^2))
-
+    res$mask=mask
+    res$s.train.mask=s.train.mask
+    res$s.train.max=s.train.max.
+        
     if(np>0) {
-        res$f.test=res$f.test+fmu
-            res$s.test.mean=apply(res$s.test, 2, mean)
-            res$s.test.mean =apply(res$s.test, 2, mean)
-        res$f.test.mean =apply(res$f.test, 2, mean)
-
+        res$f.test=res$f.test[s.train.mask, ]+fmu
+        res$f.test.mean=apply(res$f.test, 2, mean)
+        res$s.test=res$s.test[s.train.mask, ]
+        res$s.test.mean=apply(res$s.test, 2, mean)
         res$x.test=t(xp)
     }
 
@@ -254,22 +313,10 @@ if(K>0) {
     res$x.train=t(x)
     res$ntree=ntree[1]
     res$ntree[2]=ntree[2]
-    res$ndpost=nd
+    res$ndpost=ndpost
     res$xicuts=xicuts
     res$fmu = fmu
 
-    summarystats=(nd>=2)
-    if(summarystats) {
-        dimnames(res$f.varcount)[[2]]=names(xicuts)
-        res$f.varcount[2:ndpost, ]=res$f.varcount[2:ndpost, ]-res$f.varcount[1:(ndpost-1), ]
-        res$f.varcount.mean=apply(res$f.varcount, 2, mean)
-        res$f.varprob=res$f.varcount.mean/sum(res$f.varcount.mean)
-            dimnames(res$s.varcount)[[2]]=names(xicuts)
-            res$s.varcount[2:ndpost, ]=res$s.varcount[2:ndpost, ]-res$s.varcount[1:(ndpost-1), ]
-            res$s.varcount.mean=apply(res$s.varcount, 2, mean)
-            res$s.varprob=res$s.varcount.mean/sum(res$s.varcount.mean)
-    }
-    
     if(drawDPM>0) {
         res$prior = prior
         res$hyper = hyper
@@ -278,14 +325,14 @@ if(K>0) {
         res$phi = phi
 
         H=max(res$dpn.)
-        dpwt. = matrix(0, nrow=ndpost, ncol=H)
+        dpwt. = matrix(0, nrow=nd, ncol=H)
         for(h in 1:H) dpwt.[ , h]=apply(res$dpC==h, 1, sum)
         res$dpwt.=dpwt./n
 
         if(H<n) { ## it has to be, but just in case 
             res$dpmu.=res$dpmu.[ , -((H+1):n)]
             res$dpsd.=res$dpsd.[ , -((H+1):n)]
-            for(i in 1:ndpost) {
+            for(i in 1:nd) {
                 h=res$dpn.[i]
                 if(h<H) for(j in (h+1):H) {
                             res$dpmu.[i, j]=0
@@ -302,17 +349,17 @@ if(K>0) {
     ## cpo = 1/apply(1/dnorm(res$z.train, mu., sd.), 2, mean)
     ## res$lpml = sum(log(cpo))
     
-    z=matrix(log(times), nrow=ndpost, ncol=n, byrow=TRUE)
-    delta=matrix(delta, nrow=ndpost, ncol=n, byrow=TRUE)
-    cpo=(delta==2)*pnorm(z, mu., sd.)+(delta==0)*pnorm(z, mu., sd., FALSE)+
+    z=matrix(log(times), nrow=nd, ncol=n, byrow=TRUE)
+    delta=matrix(delta, nrow=nd, ncol=n, byrow=TRUE)
+    tmp=(delta==2)*pnorm(z, mu., sd.)+(delta==0)*pnorm(z, mu., sd., FALSE)+
         (delta==1)*dnorm(z, mu., sd.)
-    cpo = 1/apply(1/cpo, 2, mean)
-    res$LPML = sum(log(cpo))
+    res$CPO = 1/apply(1/tmp, 2, mean)
+    res$LPML = sum(log(res$CPO))
 
-        res$pred=predict(res, res$x.train, tc=tc,
-                         XPtr=FALSE, soffset=0) 
-        res$soffset=0.5*log(mean(res$pred$s.test.mean^2)/
-                            mean(res$s.train.mean^2)) ## for stability
+    res$pred=predict(res, res$x.train, tc=tc, XPtr=FALSE,
+                     soffset=0, probs=probs, na.rm=na.rm) 
+    res$soffset=0.5*log(mean(res$pred$s.test.mean^2)/
+                        mean(res$s.train.mean^2)) ## for stability
     
     res$drawDPM=drawDPM
     res$aft=aft
